import * as React from 'react';
import {SchemaRepository} from "./schema-repo";
import {AutoFormEvent, DisplayMode, Entry, entrySelector} from "./form-entry";
import {SchemaProperty} from "./schema-property";
import {JsonPatchBuilder} from "./json-patch-builder";

export type AutoFormOnChangeType = (e: AutoFormEvent)=>void;

export interface AutoFormProps extends React.HTMLAttributes<HTMLFormElement>{
    repo:SchemaRepository,
    schemaId:string,
    displayMode:DisplayMode,
    formData:any,
    onAutoViewChange?: AutoFormOnChangeType,
    entryKey?: string,
}

export function newSchemaRepository():SchemaRepository {
    return new SchemaRepository();
}

export function fieldValueSelector(key: string):string {
    return `form_value_${key}`;
}

export function fieldLabelSelector(key: string):string {
    return `form_label_${key}`;
}

export function autoFormSelector(key: string):string {
    return `auto_form_${key}`;
}

export interface AutoFormContext {
    isInternal:boolean;
}


export class AutoForm extends React.Component<AutoFormProps, {}> {
    public static defaultProps = {
        entryKey: "ROOT",
    };

    context: AutoFormContext;

    public static contextTypes = {
        isInternal: ()=>null,
    };

    public static childContextTypes = {
        isInternal: ()=>null,
    };

    constructor(props:AutoFormProps) {
        super(props);
    }

    getChildContext():AutoFormContext {
        return {isInternal:true};
    }

    private onAutoFormChange = (e: AutoFormEvent):void=>{
        if (this.props.onAutoViewChange) {
            const patch: JsonPatchBuilder = e.jsonPatch;

            if (patch){
                const propertyName = e.key;

                patch.prepandToPath(propertyName, this.props.formData[propertyName]);

                this.props.onAutoViewChange({
                    jsonPatch: patch,
                    key: this.props.entryKey!,
                });
            }else{
                const patch: JsonPatchBuilder = new JsonPatchBuilder();
                patch.initPath(e.key, e.value,this.props.formData);
                this.props.onAutoViewChange({
                    jsonPatch: patch,
                    key: this.props.entryKey!,
                });
            }
        }
    };

    private renderChildren() {

        if (!this.props.repo) {
            return [AutoForm.errorLabel()];
        } else {
            const validationErrors = this.props.repo.validate(this.props.formData , this.props.schemaId).errors;
            const erroneousProps = validationErrors.map((err) => err.schema as SchemaProperty);
            return this.props.repo.mapOverProperties(this.props.schemaId, makeFormEntry(this.props.formData, this.props.displayMode,this.props.repo, erroneousProps,this.onAutoFormChange));
        }
    }

    public static errorLabel() {
        return <label data-automation-id="errMessage" key="errMessage">can't render form - missing properties</label>;
    }

    render() {
        const { repo, schemaId, displayMode, formData,entryKey,onAutoViewChange,...rest }:any = this.props;   //remove props that don't need to pass to div
        let children: JSX.Element[] = this.renderChildren();

        if (!this.context.isInternal) {
            return <form {...rest}>
                {children}
            </form>
        } else {
            return <fieldset {...rest}>
                {children}
            </fieldset>;
        }
    }
}

export function makeFormEntry(formData:any, displayMode:DisplayMode,repo: SchemaRepository,erroneousProps:Array<SchemaProperty> = [],onAutoFormChange:AutoFormOnChangeType) {

    return (property:SchemaProperty, key:string):JSX.Element => {

        const isValid = (erroneousProps.find((prop) => {
                return prop == property;
            }))==undefined;

        let schemaId:string = "";
        if (property.type=='object'){
            schemaId = property['$ref'];
        }

        if (property.type=='array' && property.items && property.items.type=='object'){
            schemaId = property.items['$ref'];
        }

        return <Entry
            repo={repo}
            schemaId={schemaId}
            data-automation-id={entrySelector(key)}
            key={key}
            entryKey={key}
            title={property.title || ""}
            displayMode={displayMode}
            type={property.type}
            value={formData[key]}
            onChange={onAutoFormChange}
            valid={isValid}
        />
    }
}


